// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2012 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;

using Smdn.Formats.Mime.Formatting;

namespace Smdn.Formats.Mime {
  public class MimeHeader : IHeaderFieldBody {
    internal const char NameBodyDelimiter = ':';

    public string Name {
      get { return name; }
    }

    internal int Index {
      get; set;
    }

    public string Value {
      get; internal set;
    }

    public MimeEncodingMethod Encoding {
      get { return encoding; }
    }

    public Encoding Charset {
      get { return charset; }
    }

    internal MimeHeader(string name, string @value)
      : this(name, @value, MimeEncodingMethod.None, null)
    {
    }

    internal MimeHeader(string name, string @value, MimeEncodingMethod encoding, Encoding charset)
    {
      this.name = name;
      this.Index = 0;
      this.Value = @value;
      this.encoding = encoding;
      this.charset = charset;
    }

    public bool IsNameEquals(string name)
    {
      return string.Equals(this.name, name, StringComparison.OrdinalIgnoreCase);
    }

    public string GetParameter(string name)
    {
      return GetParameter(name, true);
    }

    public string GetParameter(string name, bool dequote)
    {
      if (name == null)
        throw new ArgumentNullException("name");

      if (Value == null)
        return null;

      name = name.ToLowerInvariant();

      var paramStarts = Value.IndexOf(name + "=", StringComparison.OrdinalIgnoreCase);

      if (paramStarts < 0)
        return null;

      var valueStarts = paramStarts + name.Length + 1;
      var valueEnds = Value.IndexOf(';', valueStarts + 1);
      string val;

      if (valueEnds == -1)
        val = Value.Substring(valueStarts);
      else
        val = Value.Substring(valueStarts, valueEnds - valueStarts).Trim();

      // if quoted
      if (dequote &&
          val.StartsWith("\"", StringComparison.Ordinal) &&
          val.EndsWith("\"", StringComparison.Ordinal))
        val = val.Substring(1, val.Length - 2);

      return val;
    }

    public string GetValueWithoutParameter()
    {
      if (Value == null)
        return null;

      var delim = Value.IndexOfAny(new[] {';', ' '});

      if (delim < 0)
        return Value;
      else
        return Value.Substring(0, delim);
    }

    public override string ToString()
    {
      return string.Concat(name,
                           MimeHeaderBuilder.NameBodyFormattingDelimiter,
                           Value);
    }

    private readonly string name;
    private readonly MimeEncodingMethod encoding;
    private readonly Encoding charset;
  }
}
