// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2012 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace Smdn.Formats.Mime {
  public class MimeHeaderCollection : ReadOnlyCollection<MimeHeader> {
    public MimeHeader this [string name] {
      get{ return headers[name]; }
    }

    internal MimeHeaders Headers {
      get { return headers; }
    }

    internal MimeHeaderCollection(MimeHeaders headers)
      : base(headers)
    {
      this.headers = headers;
    }

    public bool Contains(string name)
    {
      return headers.Contains(name);
    }

    public IEnumerable<MimeHeader> GetHeaders(string name)
    {
      foreach (var header in headers) {
        if (header.IsNameEquals(name))
          yield return header;
      }
    }

    private MimeHeaders headers;
  }

  internal class MimeHeaders : KeyedCollection<string, MimeHeader> {
    public MimeHeaders()
      : base(StringComparer.OrdinalIgnoreCase)
    {
    }

    public MimeHeaders(IEnumerable<MimeHeader> headers)
      : base(StringComparer.OrdinalIgnoreCase)
    {
      if (headers == null)
        throw new ArgumentNullException("headers");

      foreach (var header in headers) {
        Add(header);
      }
    }

    protected override void InsertItem(int index, MimeHeader item)
    {
      if (item == null)
        throw new ArgumentNullException("item");

      base.InsertItem(index, UpdateIndex(item));
    }

    protected override void SetItem(int index, MimeHeader item)
    {
      if (item == null)
        throw new ArgumentNullException("item");

      base.SetItem(index, UpdateIndex(item));
    }

    protected override string GetKeyForItem(MimeHeader item)
    {
      if (item.Index == 0)
        return item.Name;
      else
        // use ':' as a name-index delimiter
        return string.Format("{0}{1}{2}", item.Name, MimeHeader.NameBodyDelimiter, item.Index);
    }

    private MimeHeader UpdateIndex(MimeHeader item)
    {
      while (Contains(GetKeyForItem(item))) {
        item.Index++;
      }

      return item;
    }
  }
}
