using System;
using System.Collections.Generic;
#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using NUnit.Framework;

namespace Smdn.Formats.Mime {
  [TestFixture]
  public class MimeHeaderCollectionTests {
    [Test]
    public void TestIndexer()
    {
      var mime = MimeMessage.LoadMessage(@"MIME-Version: 1.0
Content-Type: text/plain
Subject: test

content"
      );

      Assert.AreEqual(3, mime.Headers.Count);

      Assert.IsNotNull(mime.Headers[0]);
      Assert.IsNotNull(mime.Headers[1]);
      Assert.IsNotNull(mime.Headers[2]);

      Assert.Throws<ArgumentOutOfRangeException>(delegate {
        var header = mime.Headers[3];
      });

      Assert.IsNotNull(mime.Headers["Content-Type"]);
      Assert.AreSame(mime.Headers["Content-Type"], mime.Headers["CONTENT-TYPE"]);

      Assert.Throws<KeyNotFoundException>(delegate {
        var header = mime.Headers["non-existent"];
      });
    }

    [Test]
    public void TestContains()
    {
      var mime = MimeMessage.LoadMessage(@"MIME-Version: 1.0
Content-Type: text/plain
Subject: test

content");

      Assert.IsTrue(mime.Headers.Contains("Content-Type"));
      Assert.IsTrue(mime.Headers.Contains("CONTENT-TYPE"));
      Assert.IsTrue(mime.Headers.Contains("Subject"));
      Assert.IsTrue(mime.Headers.Contains("subject"));
      Assert.IsFalse(mime.Headers.Contains("Date"));
    }

    [Test]
    public void TestGetHeaders()
    {
      var mime = MimeMessage.LoadMessage(@"MIME-Version: 1.0
Received: #1
Received: #2
Received: #3

content");

      CollectionAssert.IsNotEmpty(mime.Headers.GetHeaders("MIME-Version"));
      Assert.AreEqual(1, mime.Headers.GetHeaders("MIME-Version").Count());
      Assert.AreEqual("1.0", mime.Headers.GetHeaders("MIME-Version").First().Value);
      Assert.AreEqual("1.0", mime.Headers.GetHeaders("mime-version").First().Value);

      CollectionAssert.IsEmpty(mime.Headers.GetHeaders("non-existent"));

      CollectionAssert.AllItemsAreUnique(mime.Headers.GetHeaders("Received"));
      Assert.AreEqual(3, mime.Headers.GetHeaders("Received").Count());
      Assert.AreEqual(1, mime.Headers.GetHeaders("Received").Where(h => h.Value == "#1").Count(), "#1");
      Assert.AreEqual(1, mime.Headers.GetHeaders("Received").Where(h => h.Value == "#2").Count(), "#2");
      Assert.AreEqual(1, mime.Headers.GetHeaders("Received").Where(h => h.Value == "#3").Count(), "#2");
    }
  }
}
